import java.util.*;

class BNetToDot
{
    protected static void abbreviate(Vector initials)
    {
	for (int i = 0; i < initials.size(); i++)
	{
	    String name = (String)initials.elementAt(i);
	    StringBuffer abbr = new StringBuffer();

	    boolean ambiguous = true;
	    for (int j = 0; j < name.length() && (j < 2 || ambiguous); j++)
	    {
		abbr.append(name.charAt(j));
		ambiguous = false;
		for (int k = 0; k < initials.size(); k++)
		{
		    String name2 = (String)initials.elementAt(k);
		    if (k != i && 
			name2.substring(0, j+1).compareTo(abbr.toString()) == 0)
			ambiguous = true;
		}
	    }
	    initials.setElementAt(abbr.toString(), i);
	}
    }
    
    public static String toDotString(BNet bnet, boolean cpt)
    {
	StringBuffer dot = new StringBuffer();

	dot.append("digraph bnet {\n");
	dot.append("\tnode [ style=bold ];\n");
	if (bnet.hasCPTs() && cpt)
	    dot.append("\tedge [ weight=15 ];\n");

	for (int i = 0; i < bnet.size(); i++)
	{
	    dot.append("\tsubgraph pair" + i + " {\n");
	    dot.append("\t\tnode [ label = \"" + 
		       bnet.getName(i) + "\" ]; " +
		       i + ";\n");
	    
	    if (bnet.getCPT(i) != null && cpt)
	    {
		dot.append("\t\tnode [ style=solid shape=record ");
		dot.append("fontsize=9];\n");
		dot.append("\t\tnode [ label = \"");
		if (bnet.getParentConfigurations(i) > 1)
		{
		    Vector initials = new Vector();
		    for (int k = 0; k < bnet.size(); k++)
			if (bnet.isArc(k, i))
			    initials.add(bnet.getName(k));
		    abbreviate(initials);
		    
		    int [][] context = bnet.getContexts(i);
		    for (int j = 0; j < initials.size(); j++)
		    {
			dot.append("{" + 
				   (String)initials.elementAt(j) + "| ");
			for (int k = 0; k < bnet.getParentConfigurations(i);
			     k++)
			{
			    if (k> 0)
				dot.append("| ");
			    dot.append(context[k][j]);
			}
			dot.append("} |");
		    }
		}

		for (int j = 0; j < bnet.getValues(i); j++)
		{
		    if (j > 0)
			dot.append("| ");
		    dot.append("{" + j + "| ");
		    
		    for (int k = 0; k < bnet.getParentConfigurations(i);
			 k++)
		    {
			if (k > 0)
			{
			    if (bnet.getCPT(i).length == 1) 
				dot.append("| "); // one row only
			    else
				dot.append("|  "); // several -> more space
			}
			dot.append(bnet.getCPT(i)[k][j]);
		    }
		    dot.append("}");
		}
		
		dot.append("\" ]; cpt_" + i + ";\n");
		dot.append("\t\tcpt_" + i + " -> " + i + ";\n");
	    }

	    dot.append("\t};\n");
        }

	dot.append("\n\tedge [ style=bold weight=1 ];\n");

	for (int i = 0; i < bnet.size(); i++)
	    for (int j = 0; j < bnet.size(); j++)
		if (bnet.isArc(i, j))
		    dot.append("\t" + i + " -> " + 
			       j + ";\n");
	dot.append("}\n");

	return dot.toString();
    }

    public static void main(String args[])
    {
	BNet bnet;
	boolean cpt = true;

	if (args.length < 1)
	{
	    System.err.println("specify file-name.");
	    return;
	}
	else if (args.length > 1 && args[0].compareTo("--nocpt")==0)
	    cpt = false;

	bnet = BNet.readFromFile(args[args.length-1]);

	System.out.println(toDotString(bnet, cpt));
    }
}
